-----------------------------------------------------------------------------------
-- $Id: red_pitaya_top.vhd 
--
-- @brief Red Pitaya TOP module. It connects external pins and PS part with 
--        other application modules. 
--
-- @Author Matej Oblak, VHDL translation Andrej Trost (FE)
--
-- (c) Red Pitaya  http://www.redpitaya.com
 
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity red_pitaya_1 is
 Port ( 
   FIXED_IO_mio: inout std_logic_vector(53 downto 0);
   FIXED_IO_ps_clk: inout std_logic;
   FIXED_IO_ps_porb: inout std_logic;
   FIXED_IO_ps_srstb: inout std_logic;
   FIXED_IO_ddr_vrn: inout std_logic;
   FIXED_IO_ddr_vrp: inout std_logic;
   DDR_addr: inout std_logic_vector(14 downto 0);
   DDR_ba: inout std_logic_vector(2 downto 0);
   DDR_cas_n: inout std_logic;
   DDR_ck_n: inout std_logic;
   DDR_ck_p: inout std_logic;
   DDR_cke: inout std_logic;
   DDR_cs_n: inout std_logic;
   DDR_dm: inout std_logic_vector(3 downto 0);
   DDR_dq: inout std_logic_vector(31 downto 0);
   DDR_dqs_n: inout std_logic_vector(3 downto 0);
   DDR_dqs_p: inout std_logic_vector(3 downto 0);
   DDR_odt: inout std_logic;
   DDR_ras_n: inout std_logic;
   DDR_reset_n: inout std_logic;
   DDR_we_n: inout std_logic;

---- ADC
--adc_dat_a_i: in std_logic_vector(15 downto 2);
--adc_dat_b_i: in std_logic_vector(15 downto 2);
--adc_clk_p_i: in std_logic;
--adc_clk_n_i: in std_logic;
--adc_clk_o: out std_logic_vector(1 downto 0);
--adc_cdcs_o: out std_logic;

---- DAC
--dac_dat_o: out std_logic_vector(13 downto 0);
--dac_wrt_o: out std_logic;
--dac_sel_o: out std_logic;
--dac_clk_o: out std_logic;
--dac_rst_o: out std_logic;

---- PWM DAC
--dac_pwm_o: out std_logic_vector(3 downto 0);

---- XADC dodatek
--vinp_i: in std_logic_vector(4 downto 0);
--vinn_i: in std_logic_vector(4 downto 0);

---- Expansion connector
--exp_p_io: inout std_logic_vector(7 downto 0);
--exp_n_io: inout std_logic_vector(7 downto 0);


---- SATA connector
--daisy_p_o: out std_logic_vector(1 downto 0);
--daisy_n_o: out std_logic_vector(1 downto 0);
--daisy_p_i: in std_logic_vector(1 downto 0);
--daisy_n_i: in std_logic_vector(1 downto 0);

-- LED
   led_o: out std_logic_vector(7 downto 0)      
);
end red_pitaya_1;

architecture Behavioral of red_pitaya_1 is
---------------------------------------------------------------------------------
--  Connections to PS
 signal fclk: std_logic_vector(3 downto 0);       --[0]-125MHz, [1]-250MHz, [2]-50MHz, [3]-200MHz
 signal frstn: std_logic_vector(3 downto 0);

 signal ps_sys_clk: std_logic;
 signal ps_sys_rstn: std_logic;
 signal ps_sys_addr: std_logic_vector(31 downto 0);
 signal ps_sys_wdata: std_logic_vector(31 downto 0);
 signal ps_sys_sel: std_logic_vector(3 downto 0);
 signal ps_sys_wen: std_logic;
 signal ps_sys_ren: std_logic;
 signal ps_sys_rdata: std_logic_vector(31 downto 0);
 signal ps_sys_err: std_logic;
 signal ps_sys_ack: std_logic;

 component red_pitaya_ps is
  port (
   FIXED_IO_mio: inout std_logic_vector(53 downto 0);
   FIXED_IO_ps_clk: inout std_logic;
   FIXED_IO_ps_porb: inout std_logic;
   FIXED_IO_ps_srstb: inout std_logic;
   FIXED_IO_ddr_vrn: inout std_logic;
   FIXED_IO_ddr_vrp: inout std_logic;
   DDR_addr: inout std_logic_vector(14 downto 0);
   DDR_ba: inout std_logic_vector(2 downto 0);
   DDR_cas_n: inout std_logic;
   DDR_ck_n: inout std_logic;
   DDR_ck_p: inout std_logic;
   DDR_cke: inout std_logic;
   DDR_cs_n: inout std_logic;
   DDR_dm: inout std_logic_vector(3 downto 0);
   DDR_dq: inout std_logic_vector(31 downto 0);
   DDR_dqs_n: inout std_logic_vector(3 downto 0);
   DDR_dqs_p: inout std_logic_vector(3 downto 0);
   DDR_odt: inout std_logic;
   DDR_ras_n: inout std_logic;
   DDR_reset_n: inout std_logic;
   DDR_we_n: inout std_logic;

   fclk_clk_o: out std_logic_vector(3 downto 0);
   fclk_rstn_o: out std_logic_vector(3 downto 0);

   -- system read/write channel
   sys_clk_o: out std_logic;
   sys_rstn_o: out std_logic;
   sys_addr_o: out std_logic_vector(31 downto 0);
   sys_wdata_o: out std_logic_vector(31 downto 0);
   sys_sel_o: out std_logic_vector(3 downto 0);
   sys_wen_o: out std_logic;
   sys_ren_o: out std_logic;
   sys_rdata_i: in std_logic_vector(31 downto 0);
   sys_err_i: in std_logic;
   sys_ack_i: in std_logic;

   -- SPI master
   spi_ss_o: out std_logic;
   spi_ss1_o: out std_logic;
   spi_ss2_o: out std_logic;
   spi_sclk_o: out std_logic;
   spi_mosi_o: out std_logic;
   spi_miso_i: in std_logic;
   -- SPI slave
   spi_ss_i: in std_logic;
   spi_sclk_i: in std_logic;
   spi_mosi_i: in std_logic;
   spi_miso_o: out std_logic
  );
 end component;

-- HW test logic
 signal gnd: std_logic := '0';
 signal vcc: std_logic := '1';
 signal clk50: std_logic;
 signal ld: std_logic_vector(7 downto 0);
 signal cnt: std_logic_vector(23 downto 0);

begin
  
i_ps: red_pitaya_ps port map ( 
  FIXED_IO_mio => FIXED_IO_mio,
  FIXED_IO_ps_clk => FIXED_IO_ps_clk, 
  FIXED_IO_ps_porb => FIXED_IO_ps_porb,
  FIXED_IO_ps_srstb => FIXED_IO_ps_srstb,
  FIXED_IO_ddr_vrn  => FIXED_IO_ddr_vrn,
  FIXED_IO_ddr_vrp => FIXED_IO_ddr_vrp,
  DDR_addr => DDR_addr,
  DDR_ba => DDR_ba,
  DDR_cas_n => DDR_cas_n,
  DDR_ck_n => DDR_ck_n,
  DDR_ck_p => DDR_ck_p,
  DDR_cke => DDR_cke,
  DDR_cs_n => DDR_cs_n,
  DDR_dm => DDR_dm,
  DDR_dq => DDR_dq,
  DDR_dqs_n => DDR_dqs_n,
  DDR_dqs_p => DDR_dqs_p,
  DDR_odt => DDR_odt,
  DDR_ras_n => DDR_ras_n,
  DDR_reset_n => DDR_reset_n,
  DDR_we_n => DDR_we_n,
  fclk_clk_o => fclk,  
  fclk_rstn_o => frstn,

  -- system read/write channel
  sys_clk_o => ps_sys_clk,
  sys_rstn_o => ps_sys_rstn,
  sys_addr_o => ps_sys_addr,
  sys_wdata_o => ps_sys_wdata,
  sys_sel_o => ps_sys_sel,
  sys_wen_o => ps_sys_wen,
  sys_ren_o => ps_sys_ren,
  sys_rdata_i => ps_sys_rdata,
  sys_err_i => ps_sys_err,
  sys_ack_i => ps_sys_ack,

  -- SPI master
  spi_ss_o => open,
  spi_ss1_o => open,
  spi_ss2_o => open,
  spi_sclk_o => open,
  spi_mosi_o => open,
  spi_miso_i => gnd,

  -- SPI slave
  spi_ss_i => vcc,
  spi_sclk_i => gnd,
  spi_mosi_i => gnd,
  spi_miso_o => open
);

clk50 <= fclk(2);
p: process(clk50)
begin
 if rising_edge(clk50) then
   cnt <= cnt + 1;
   if cnt=0 then
     ld <= not ld;
   end if;
 end if;
end process;

led_o <= ld;

end Behavioral;
